﻿using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using Abt.Controls.SciChart.Model.DataSeries;

namespace WpfApplication1
    {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window, INotifyPropertyChanged
        {
        #region INotifyPropertyChanged Implementation

        [field: NonSerialized]
        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged(String info)
            {
            if (PropertyChanged != null)
                {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
                }
            }

        #endregion INotifyPropertyChanged Implementation

        #region Bindable HeatMap DataSeries

        private Heatmap2DArrayDataSeries<int, int, float> FixationHeatMapDataSeries;

        [NonSerialized]
        private float[,] HeatMapVoxels;

        [Browsable(false)]
        public Heatmap2DArrayDataSeries<int, int, float> _CurrentHeatMapDataSeries
            {
            get
                {
                return FixationHeatMapDataSeries;
                }
            }

        #endregion Bindable HeatMap DataSeries

        public MainWindow()
            {
            InitializeComponent();
            HeatMapVoxels = new float[200, 200];
            }

        private void Start_Button_Click(object sender, RoutedEventArgs e)
            {
            var MainWindowSycnContext=SynchronizationContext.Current;
            for (int i = 0; i < 6; i++)
                {
                Thread StatusScreenUIThread = new Thread(new ParameterizedThreadStart(PopupWindowThreadFunc));
                StatusScreenUIThread.SetApartmentState(ApartmentState.STA);
                StatusScreenUIThread.IsBackground = true;
                StatusScreenUIThread.Name = "Status Screen UI Thread" + i;
                StatusScreenUIThread.Priority = ThreadPriority.Normal;                
                StatusScreenUIThread.Start(MainWindowSycnContext);
                WindowIsReadySignal.WaitOne();
                }

            DispatcherTimer _timer = new DispatcherTimer()
                {
                    Interval = TimeSpan.FromMilliseconds(100),
                    IsEnabled = true
                };
            _timer.Tick += _timer_Tick;
            }


        /// <summary>
        /// Updating of the hearmap values
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _timer_Tick(object sender, EventArgs e)
            {
            Random R = new Random();
            for (int iy = 0; iy < 200; iy++)
                for (int ix = 0; ix < 200; ix++)
                    {
                    HeatMapVoxels[iy, ix] = (float)(R.NextDouble() * 100);
                    }

            FixationHeatMapDataSeries = new Heatmap2DArrayDataSeries<int, int, float>(HeatMapVoxels, ix => ix, iy => iy);
            NotifyPropertyChanged("_CurrentHeatMapDataSeries");
            }


        private AutoResetEvent WindowIsReadySignal = new AutoResetEvent(false);

        private void PopupWindowThreadFunc(object o)
            {

            /// Create our context, and install it or set the context of the main window (commented line)
            SynchronizationContext.SetSynchronizationContext(
                new DispatcherSynchronizationContext(
                    Dispatcher.CurrentDispatcher));

            // SynchronizationContext.SetSynchronizationContext(o as DispatcherSynchronizationContext);
            


            PopupWindow W = new PopupWindow();
            W.Closed += (s, e) =>
                Dispatcher.CurrentDispatcher.BeginInvokeShutdown(DispatcherPriority.Background);

            W.DataContext = this;
            W.Show();

            System.Windows.Forms.Application.DoEvents();/// Wait for window completing rendering                                                                  /// 

            Dispatcher.CurrentDispatcher.BeginInvoke((Action)delegate()
                        {
                        WindowIsReadySignal.Set(); /// signals to the main thread that the new window is ready
                        });

            System.Windows.Threading.Dispatcher.Run();
            }
        }

    class MySynchronizationProvider : System.Threading.SynchronizationContext {
            public MySynchronizationProvider() {
                base.SetWaitNotificationRequired();
            }
            public override int Wait(IntPtr[] waitHandles, bool waitAll, int millisecondsTimeout) {
                for (; ; ) {
                    int result = MsgWaitForMultipleObjects(waitHandles.Length, waitHandles, waitAll, millisecondsTimeout, 8);
                    if (result == waitHandles.Length) System.Windows.Forms.Application.DoEvents();
                    else return result;
                }
            }
            [DllImport("user32.dll")]
            private static extern int MsgWaitForMultipleObjects(int cnt, IntPtr[] waitHandles, bool waitAll,
                int millisecondTimeout, int mask);        
        }

    }